﻿using System;

using HeuristicLab.Core;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Views {
  [View("ParsedConstraint View")]
  [Content(typeof(ParsedConstraint), true)]
  public partial class ParsedConstraintView : AsynchronousContentView {

    public new ParsedConstraint Content {
      get => (ParsedConstraint)base.Content;
      set => base.Content = value;
    }

    public ParsedConstraintView() {
      InitializeComponent();
      errorOutput.Text = "";
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        constraintsInput.Text = string.Empty;
        errorOutput.Text = "";
      } else {
        constraintsInput.Text = Content.Input;
        constraintsOutput.Content = new ItemList<IntervalConstraint>(Content.Constraints) ?? new ItemList<IntervalConstraint>();
      }
    }

    private ItemList<IntervalConstraint> ParseConstraints(string input) {
      ItemList<IntervalConstraint> constraints = new ItemList<IntervalConstraint>(IntervalConstraintsParser.ParseInput(input, Content.TargetVariable, Content.AllowedVariables));
      return constraints;
    }

    private void parseBtn_Click(object sender, EventArgs e) {
      if (constraintsInput.Text != null) {
        var constaints = ParseConstraints(constraintsInput.Text);
        constraintsOutput.Content = constaints;
        Content.Constraints = constaints;
        Content.Input = constraintsInput.Text;
        errorOutput.Text = "";
      } else {
        errorOutput.Text = "No constraints were found!";
      }
    }

    private void constraintsInput_TextChanged(object sender, EventArgs e) {
      errorOutput.Text = "Unparsed changes! Press parse button to save changes.";
    }
  }
}
